"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var state_1 = require("./state");
var MuRemoteClientState = (function () {
    function MuRemoteClientState(client, schema, windowSize) {
        this.tick = 0;
        this._client = client;
        this.sessionId = client.sessionId;
        this.windowSize = windowSize;
        this.state = schema.clone(schema.identity);
        this.history = new state_1.MuStateSet(this.state);
    }
    MuRemoteClientState.prototype.close = function () {
        this._client.close();
    };
    return MuRemoteClientState;
}());
exports.MuRemoteClientState = MuRemoteClientState;
function findClient(clients, id) {
    for (var i = 0; i < clients.length; ++i) {
        if (clients[i].sessionId === id) {
            return i;
        }
    }
    return -1;
}
function removeItem(array, index) {
    array[index] = array[array.length - 1];
    array.pop();
}
var MuServerState = (function () {
    function MuServerState(spec) {
        this.clients = [];
        this.tick = 0;
        this.windowSize = 0;
        this.maxHistorySize = Infinity;
        this._observedStates = [];
        this.server = spec.server;
        this.schema = spec.schema;
        if (typeof spec.windowSize === 'number') {
            this.windowSize = spec.windowSize;
        }
        if (typeof spec.maxHistorySize === 'number') {
            this.maxHistorySize = spec.maxHistorySize;
        }
        this.state = this.schema.server.clone(this.schema.server.identity);
        this.history = new state_1.MuStateSet(this.schema.server.identity);
        this._protocol = spec.server.protocol(state_1.MuDefaultStateSchema);
    }
    MuServerState.prototype.configure = function (spec) {
        var _this = this;
        this._protocol.configure({
            message: {
                ackState: function (client, tick) {
                    var index = findClient(_this.clients, client.sessionId);
                    state_1.addObservation(_this._observedStates[index], tick);
                },
                forgetState: function (client, tick) {
                    var index = findClient(_this.clients, client.sessionId);
                    state_1.forgetObservation(_this._observedStates[index], tick);
                },
            },
            ready: function () {
                if (spec && spec.ready) {
                    spec.ready();
                }
            },
            raw: function (client_, data, unreliable) {
                if (!(data instanceof Uint8Array)) {
                    return;
                }
                var client = _this.clients[findClient(_this.clients, client_.sessionId)];
                if (state_1.parseState(data, _this.schema.client, client, client_.message.ackState, client_.message.forgetState)) {
                    if (spec && spec.state) {
                        spec.state(client, client.state, client.tick, !unreliable);
                    }
                }
            },
            connect: function (client_) {
                var client = new MuRemoteClientState(client_, _this.schema.client, _this.windowSize);
                _this.clients.push(client);
                _this._observedStates.push([0]);
                if (spec && spec.connect) {
                    spec.connect(client);
                }
                state_1.publishState(_this.schema.server, _this._observedStates, _this, client_.sendRaw, true);
            },
            disconnect: function (client_) {
                var clientId = findClient(_this.clients, client_.sessionId);
                if (spec && spec.disconnect) {
                    spec.disconnect(_this.clients[clientId]);
                }
                var client = _this.clients[clientId];
                if (client) {
                    state_1.garbageCollectStates(_this.schema.client, client.history, Infinity);
                }
                removeItem(_this.clients, clientId);
                removeItem(_this._observedStates, clientId);
            },
        });
    };
    MuServerState.prototype.commit = function (reliable) {
        var observedStates = this._observedStates;
        var mostRecentCommonTick = state_1.publishState(this.schema.server, observedStates, this, this._protocol.broadcastRaw, !!reliable);
        for (var i = 0; i < observedStates.length; ++i) {
            var ticks = observedStates[i];
            if (this.tick - ticks[ticks.length - 1] >= this.maxHistorySize) {
                console.log("kick client " + this.clients[i].sessionId + " with stale state");
                this.clients[i].close();
                continue;
            }
            var ptr = 1;
            while (ptr < ticks.length && ticks[ptr] < mostRecentCommonTick) {
                ++ptr;
            }
            var optr = 1;
            while (ptr < ticks.length) {
                ticks[optr++] = ticks[ptr++];
            }
            ticks.length = optr;
        }
    };
    return MuServerState;
}());
exports.MuServerState = MuServerState;
//# sourceMappingURL=server.js.map